/* reglib.h
 * Written by Joshua Rowe
 */

#ifndef __REGLIB_CONNECT_H__
#define	__REGLIB_CONNECT_H__

#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<netdb.h>
#include	<netinet/in.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<unistd.h>

#define	RG_ERR_SUCCESS	0
#define	RG_ERR		0x0400
#define	RG_ERR_NOHOST	0x0400
#define	RG_ERR_INVREQ	0x0401
#define	RG_ERR_NOMORE	0x0402
#define	RG_ERR_NONAME	0x0403
#define	RG_ERR_NOACC	0x0404
#define	RG_ERR_BADPASS	0x0405
#define	RG_ERR_EXIST	0x0406

extern	char *	reglib_errs[];

#define	RGQ_FIRSTKEY	1
#define	RGQ_NEXTKEY	2
#define	RGQ_LOGIN	3
#define	RGQ_CREATEKEY	4
#define	RGQ_EXISTKEY	5
#define	RGQ_GETKEYI	6
#define	RGQ_KILLKEY	7
#define	RGQ_FIRSTVAL	8
#define	RGQ_NEXTVAL	9
#define	RGQ_SETKEYI	10
#define	RGQ_KILLVAL	11
#define	RGQ_SETVAL	12
#define	RGQ_GETVAL	13
#define	RGQ_EXISTVAL	14

#define	RGT_INT		1
#define	RGT_STRING	2
#define	RGT_BIN		3

enum	RKM	{
  RKM_STICKY	= 01000,
  RKM_U		= 6,
  RKM_UR	= 00400,
  RKM_UW	= 00200,
  RKM_UX	= 00100,
  RKM_G		= 3,
  RKM_GR	= 00040,
  RKM_GW	= 00020,
  RKM_GX	= 00010,
  RKM_O		= 0,
  RKM_OR	= 00004,
  RKM_OW	= 00002,
  RKM_OX	= 00001,
  RKM_DEF	= 00755,
  RKM_FULL	= 00777,
  RKM_A		= 00007,
  RKM_R		= 00004,
  RKM_W		= 00002,
  RKM_X		= 00001
  };

extern	int	reglib_errno;

struct	reg_keyi
{
  char *	name;
  int		uid;
  int		gid;
  int		mode;
};

__BEGIN_DECLS

int	regd_connect(const char *hostname, unsigned short int port);
int	regd_createkey(int sock, const char *path, int mode);
int	regd_createkeyr(int sock, const char *path, int mode);
const char *	regd_err(int err);
int	regd_existkey(int sock, const char *path);
int	regd_existval(int sock, const char *path);
char *	regd_getfirstkey(int sock, const char *path);
char *	regd_getfirstval(int sock, const char *path);
int	regd_getkeyinfo(int sock, const char *path, struct reg_keyi *k);
char *	regd_getnextkey(int sock, const char *path);
char *	regd_getnextval(int sock, const char *path);
char *	regd_getval(int sock, const char *path);
char *	regd_getvalv(int sock, const char *path, const char *valname);
char *	regd_getvalvd(int sock, const char *path, const char *valname,
		      const char *defval);
int	regd_getvals(int sock, const char *path, const char *t, ...);
int	regd_getvalsd(int sock, const char *path, const char *t,
		      const char *defval, ...);
int	regd_getvalsv(int sock, const char *path, const char *valname,
		      const char *t, ...);
int	regd_getvalsvd(int sock, const char *path, const char *valname,
		      const char *defval, const char *t, ...);
int	regd_getvalsr(int sock, const char *path, const char *t, ...);
int	regd_getvalsdr(int sock, const char *path, const char *t,
		      const char *defval, ...);
int	regd_getvalsvr(int sock, const char *path, const char *valname,
		      const char *t, ...);
int	regd_getvalsvdr(int sock, const char *path, const char *valname,
		      const char *defval, const char *t, ...);
int	regd_initsockaddr(struct sockaddr_in *name, const char *hostname,
			  unsigned short int port);
int	regd_killkey(int sock, const char *path);
int	regd_killval(int sock, const char *path);
int	regd_login(int sock, char *aname, char *apwd);
int	regd_readint(int sock);
char *	regd_readstring(int sock);
int	regd_setkeyinfo(int sock, const char *path, int mode,
			int uid, int gid);
int	regd_setkeymode(int sock, const char *path, int mode);
int	regd_setval(int sock, const char *path, const char *val);
int	regd_setvalv(int sock, const char *path, const char *valname,
		     const char *val);
int	regd_setvals(int sock, const char *path, const char *t, ...);
int	regd_setvalsv(int sock, const char *path, const char *valname,
		      const char *t, ...);
int	regd_setvalr(int sock, const char *path, const char *val);
int	regd_setvalvr(int sock, const char *path, const char *valname,
		     const char *val);
int	regd_setvalsr(int sock, const char *path, const char *t, ...);
int	regd_setvalsvr(int sock, const char *path, const char *valname,
		      const char *t, ...);
void	regd_writeint(int sock, int val);
void	regd_writestring(int sock, const char *string);
void	regd_writetrans(int sock, int req, const char *name, int type, int len,
			const void *data);


__END_DECLS

#endif

