/* test.c */
/* Written by Joshua Rowe */
/* tests the capabilities of the clientlib */

#include	"reglib.h"
#include	<malloc.h>
#include	<stdio.h>
#include	<string.h>
#include	<termios.h>

struct	termios	saved;
struct	termios	new;

char *	getstring()
{
  char	c[32768];
  scanf("%[^\n]", c);
  fgetc(stdin);
  return	strdup(c);
}

char *	buf[10];
char	path[32768];

char *	mystrdup(const char *z)
{
  char *	c;
  c	= (char *)malloc(strlen(z) + 32768);
  strcpy(c, z);
}

void	createkey(int sock)
{
  int	mode;
  printf("Enter key name: ");
  buf[0]	= getstring();
  if (!strlen(buf[0]))
    {
      free(buf[0]);
      return;
    }
  printf("Enter mode in octal: ");
  scanf("%o", &mode);
  buf[2]	= malloc(strlen(buf[0]) + strlen(path) + 2);
  strcpy(buf[2], path);
  strcat(buf[2], "/");
  strcat(buf[2], buf[0]);
  regd_createkey(sock, buf[2], mode);
  free(buf[0]);
  free(buf[2]);
}

void	descendkey(int sock)
{
  printf("Enter key name: ");
  buf[0]	= getstring();
  strcat(path, "/");
  strcat(path, buf[0]);
  if (!regd_existkey(sock, path))
    {
      if (strrchr(path, '/'))
	*strrchr(path, '/')	= 0;
    }
  free(buf[0]);
}

void	getkeyinfo(int sock)
{
  struct reg_keyi	x;
  buf[1]	= mystrdup(path);
  printf("Enter key name: ");
  buf[0]	= getstring();
  strcat(buf[1], "/");
  strcat(buf[1], buf[0]);
  free(buf[0]);
  regd_getkeyinfo(sock, buf[1], &x);
  free(buf[1]);
  if (reglib_errno)
    return;
  printf("%s:   uid %i   gid %i   mode %o\n",
	 x.name,
	 x.uid,
	 x.gid,
	 x.mode);
  free(x.name);
}

void	getvalinfo(int sock)
{
  buf[1]	= mystrdup(path);
  printf("Enter key name: ");
  buf[0]	= getstring();
  strcat(buf[1], "|");
  strcat(buf[1], buf[0]);
  free(buf[0]);
  buf[2]	= regd_getval(sock, buf[1]);
  printf("%s=%s\n", buf[1], buf[2]);
  free(buf[1]);
  free(buf[2]);
  if (reglib_errno)
    return;
}
void	killkey(int sock)
{
  printf("Enter key name: ");
  buf[0]	= getstring();
  buf[1]	= mystrdup(path);
  strcat(buf[1], "/");
  strcat(buf[1], buf[0]);
  regd_killkey(sock, buf[1]);
  free(buf[0]);
  free(buf[1]);
}

void	killval(int sock)
{
  printf("Enter val name: ");
  buf[0]	= getstring();
  buf[1]	= mystrdup(path);
  strcat(buf[1], "|");
  strcat(buf[1], buf[0]);
  regd_killval(sock, buf[1]);
  free(buf[0]);
  free(buf[1]);
}

void	scankeys(int sock)
{
  char *	x;
  char *	y;
  x	= regd_getfirstkey(sock, path);
  while (x != 0)
    {
      printf("%s\n", x);
      y	= regd_getnextkey(sock, x);
      free(x);
      x	= y;
    }
}

void	scanvals(int sock)
{
  char *	x;
  char *	y;
  x	= regd_getfirstval(sock, path);
  while (x != 0)
    {
      printf("%s\n", x);
      y	= regd_getnextval(sock, x);
      free(x);
      x	= y;
    }
}

void	setkeyinfo(int i)
{
  int	mode, uid, gid;
  printf("Path name: ");
  buf[0]	= getstring();
  printf("New mode: ");
  buf[1]	= getstring();
  mode		= strtol(buf[1], NULL, 0);
  free(buf[1]);
  printf("New uid: ");
  buf[1]	= getstring();
  uid		= strtol(buf[1], NULL, 0);
  free(buf[1]);
  buf[2]	= malloc(strlen(path) + strlen(buf[0]) + 2);
  strcpy(buf[2], path);
  strcat(buf[2], "/");
  strcat(buf[2], buf[0]);
  free(buf[0]);
  if (uid != -1)
    {
      printf("New gid: ");
      buf[1]	= getstring();
      gid	= strtol(buf[1], NULL, 0);
      free(buf[1]);
      regd_setkeyinfo(i, buf[2], mode, uid, gid);
    }
  else
    regd_setkeymode(i, buf[2], mode);
  free(buf[2]);
}

void	setval(int x)
{
  printf("Enter val name: ");
  buf[0]	= getstring();
  buf[1]	= mystrdup(path);
  strcat(buf[1], "|");
  strcat(buf[1], buf[0]);
  printf("Enter value: ");
  buf[2]	= getstring();
  regd_setval(x, buf[1], buf[2]);
  free(buf[0]);
  free(buf[1]);
  free(buf[2]);
}

int	main(int argc, char **argv)
{
  int	x;
  char	*s;
  int	z;
  int	y;
  int	flag;
  char	c;
  flag	= 1;
  x	= -1;
  x	= regd_connect((argc > 1) ? argv[1] : "127.0.0.1", 0);
  while (x == -1)
    {
      sleep(2);
      printf("Attempting to connect...\n");
      x	= regd_connect((argc > 1) ? argv[1] : "127.0.0.1", 0);
    }
  while (flag)
    {
      printf("Login ID: ");
      buf[0]	= getstring();
      printf("Password: ");
      tcgetattr(0, &saved);
      new	= saved;
      new.c_lflag	&= ~ECHO;
      tcsetattr(0, TCSANOW, &new);
      buf[1]	= getstring();
      tcsetattr(0, TCSANOW, &saved);
      printf("Login %s\n", (flag = regd_login(x, buf[0], buf[1]))
	     ? "succeeded" : "failed");
      if (!flag)
	{
	  printf("Try again? ");
	  buf[2]	= getstring();
	  if (buf[2][0] == 'n')
	    flag	= 0;
	}
      else
	flag	= 0;
      free(buf[0]);
      free(buf[1]);
      free(buf[2]);
    }
  regd_getvalsvdr(x, "/me/software/regd", "|times-used", "0", "%i", &z);
  regd_setvalsvr(x, "/me/software/regd", "|times-used", "%i", ++z);
  printf("You have logged in using this program %i times!\n", z);
  flag	= 1;
  strcpy(path, "/me");
  while (flag)
    {
      printf(
	     "%s\n"
             "path: %s\n"
	     "        c  - Create key          C - set val\n"
	     "        d  - Descend key\n"
	     "        i  - Get key info        I - get val\n"
	     "        o  - Set key info\n"
	     "        k  - Kill key            K - kill value\n"
	     "        s  - Scan for keys       S - scan for values\n"
	     "        u  - Up 1 level\n"
	     "        x  - Quit\n"
	     "Enter choice: ",
	     regd_err(reglib_errno),
	     path
	     );
      s	= getstring();
      c	= s[0];
      free(s);
      printf("\n");
      switch(c)
	{
	case 'c':
	  createkey(x);
	  break;
	case 'C':
	  setval(x);
	  break;
	case 'd':
	  descendkey(x);
	  break;
	case 'i':
	  getkeyinfo(x);
	  break;
	case 'I':
	  getvalinfo(x);
	  break;
	case 'k':
	  killkey(x);
	  break;
	case 'K':
	  killval(x);
	  break;
	case 'o':
	  setkeyinfo(x);
	  break;
	case 's':
	  scankeys(x);
	  break;
	case 'u':
	  if (strrchr(path, '/'))
	    *strrchr(path, '/')	= 0;
	  break;
	case 'x':
	  exit(0);
	case 'S':
	  scanvals(x);
	  break;
	case '\n':
	  break;
	default:
	  printf("Error - invalid command!\n");
	  break;
	}
    }
}
