/* trans.c */
/* Written by Joshua Rowe */

#include	"reglib.h"
#include	<malloc.h>
#include	<stdarg.h>
#include	<stdio.h>
#include	<string.h>

char	reglib_suc[]	= "Successful";
char *	reglib_errs[]	= {
  "Could not connect to remote host",
  "Invalid request",
  "Last entry in list",
  "Name not found",
  "Access denied",
  "Invalid login attempt",
  "Entry already exists"
};

int	reglib_errno;

int	regd_createkey(int sock, const char *path, int mode)
{
  regd_writetrans(sock, RGQ_CREATEKEY, path, mode, 0, 0);
  return reglib_errno;
}

int    	regd_createkeyr(int sock, const char *path, int mode)
{
  char *	tmppath;
  char *	offptr;
  char		val;
  tmppath	= alloca(strlen(path) + 1);
  strcpy(tmppath, path);
  offptr	= tmppath;
  while (offptr = strpbrk(offptr + 1, "/|"))
    {
      val	= *offptr;
      *offptr	= 0;
      if (!regd_existkey(sock, tmppath))
	regd_createkey(sock, tmppath, mode);
      if (reglib_errno)
	return reglib_errno;
      *offptr	= val;
    }
  return reglib_errno;
}

const char *	regd_err(int err)
{
  if (err == 0)
    return reglib_suc;
  return reglib_errs[err - RG_ERR];
}

int	regd_existkey(int sock, const char *path)
{
  regd_writetrans(sock, RGQ_EXISTKEY, path, 0, 0, 0);
  return !reglib_errno;
}

int	regd_existval(int sock, const char *path)
{
  regd_writetrans(sock, RGQ_EXISTVAL, path, 0, 0, 0);
  return !reglib_errno;
}

char *	regd_getfirstkey(int sock, const char *path)
{
  if (path == NULL)
    path	= "";
  regd_writetrans(sock, RGQ_FIRSTKEY, path, 0, 0, NULL);
  if (reglib_errno != RG_ERR_SUCCESS)
    return NULL;
  return regd_readstring(sock);
}

char *	regd_getfirstval(int sock, const char *path)
{
  if (path == NULL)
    path	= "";
  regd_writetrans(sock, RGQ_FIRSTVAL, path, 0, 0, NULL);
  if (reglib_errno != RG_ERR_SUCCESS)
    return NULL;
  return regd_readstring(sock);
}

int	regd_getkeyinfo(int sock, const char *path, struct reg_keyi *k)
{
  regd_writetrans(sock, RGQ_GETKEYI, path, 0, 0, NULL);
  if (reglib_errno != RG_ERR_SUCCESS)
    return 0;
  k->name	= regd_readstring(sock);
  if (reglib_errno != RG_ERR_SUCCESS)
    {
      if (k->name)
	free(k->name);
      return 0;
    }
  k->uid	= regd_readint(sock);
  if (reglib_errno != RG_ERR_SUCCESS)
    {
      free(k->name);
      return 0;
    }
  k->gid	= regd_readint(sock);
  if (reglib_errno != RG_ERR_SUCCESS)
    {
      free(k->name);
      return 0;
    }
  k->mode	= regd_readint(sock);
  if (reglib_errno != RG_ERR_SUCCESS)
    {
      free(k->name);
      return 0;
    }
  return 1;
}

char *	regd_getnextkey(int sock, const char *path)
{
  if (path == NULL)
    path	= "";
  regd_writetrans(sock, RGQ_NEXTKEY, path, 0, 0, NULL);
  if (reglib_errno != RG_ERR_SUCCESS)
    return NULL;
  return regd_readstring(sock);
}

char *	regd_getnextval(int sock, const char *path)
{
  if (path == NULL)
    path	= "";
  regd_writetrans(sock, RGQ_NEXTVAL, path, 0, 0, NULL);
  if (reglib_errno != RG_ERR_SUCCESS)
    return NULL;
  return regd_readstring(sock);
}

char *	regd_getval(int sock, const char *path)
{
  if (path == NULL)
    {
      reglib_errno	= RG_ERR_NONAME;
      return;
    }
  regd_writetrans(sock, RGQ_GETVAL, path, 0, 0, 0);
  if (!reglib_errno)
    return regd_readstring(sock);
  else
    return 0;
}

char *	regd_getvalv(int sock, const char *path, const char *valname)
{
  char *	c;
  char *	x;
  c	= (char *)malloc(strlen(path) + strlen(valname) + 2);
  strcpy(c, path);
  if (valname[0] != '|')
    strcat(c, "|");
  strcat(c, valname);
  x	= regd_getval(sock, c);
  free(c);
  return x;
}

int	regd_getvals(int sock, const char *path, const char *t, ...)
{
  va_list	x;
  char *	c;
  c	= regd_getval(sock, path);
  if (c == 0)
    return reglib_errno;
  va_start(x, t);
  vsscanf(c, t, x);
  va_end(x);
  free(c);
  return reglib_errno;
}

int	regd_getvalsv(int sock, const char *path, 
		      const char *valname, const char *t, ...)
{
  char *	c;
  va_list	x;
  c	= regd_getvalv(sock, path, valname);
  if (c == 0)
    return reglib_errno;
  va_start(x, t);
  vsscanf(c, t, x);
  va_end(x);
  free(c);
  return reglib_errno;
}

char *	regd_getvald(int sock, const char *path, const char *defval)
{
  if (path == NULL)
    {
      reglib_errno	= RG_ERR_NONAME;
      return;
    }
  regd_writetrans(sock, RGQ_GETVAL, path, 0, 0, 0);
  if (reglib_errno == RG_ERR_NONAME)
    return strdup(defval);
  if (!reglib_errno)
    return regd_readstring(sock);
  else
    return 0;
}

char *	regd_getvalvd(int sock, const char *path, const char *valname,
		     const char *defval)
{
  char *	c;
  char *	x;
  c	= (char *)alloca(strlen(path) + strlen(valname) + 2);
  strcpy(c, path);
  if (valname[0] != '|')
    strcat(c, "|");
  strcat(c, valname);
  x	= regd_getvald(sock, c, defval);
  return x;
}

int	regd_getvalsd(int sock, const char *path, const char *defval, 
		      const char *t, ...)
{
  va_list	x;
  char *	c;
  c	= regd_getvald(sock, path, defval);
  if (c == 0)
    return reglib_errno;
  va_start(x, t);
  vsscanf(c, t, x);
  va_end(x);
  free(c);
  return reglib_errno;
}

int	regd_getvalsvd(int sock, const char *path, 
		       const char *valname, const char *defval,
		       const char *t, ...)
{
  char *	c;
  va_list	x;
  c	= regd_getvalvd(sock, path, valname, defval);
  if (c == 0)
    return reglib_errno;
  va_start(x, t);
  vsscanf(c, t, x);
  va_end(x);
  free(c);
  return reglib_errno;
}

char *	regd_getvaldr(int sock, const char *path, const char *defval)
{
  if (path == NULL)
    {
      reglib_errno	= RG_ERR_NONAME;
      return;
    }
  regd_writetrans(sock, RGQ_GETVAL, path, 0, 0, 0);
  if (reglib_errno == RG_ERR_NONAME)
    {
      regd_setvalr(sock, path, defval);
      return strdup(defval);
    }
  if (!reglib_errno)
    return regd_readstring(sock);
  else
    return 0;
}

char *	regd_getvalvdr(int sock, const char *path, const char *valname,
		     const char *defval)
{
  char *	c;
  char *	x;
  c	= (char *)alloca(strlen(path) + strlen(valname) + 2);
  strcpy(c, path);
  if (valname[0] != '|')
    strcat(c, "|");
  strcat(c, valname);
  x	= regd_getvaldr(sock, c, defval);
  return x;
}

int	regd_getvalsdr(int sock, const char *path, const char *defval,
		       const char *t, ...)
{
  va_list	x;
  char *	c;
  c	= regd_getvaldr(sock, path, defval);
  if (c == 0)
    return reglib_errno;
  va_start(x, t);
  vsscanf(c, t, x);
  va_end(x);
  free(c);
  return reglib_errno;
}

int	regd_getvalsvdr(int sock, const char *path, 
		       const char *valname, const char *defval,
		       const char *t, ...)
{
  char *	c;
  va_list	x;
  c	= regd_getvalvdr(sock, path, valname, defval);
  if (c == 0)
    return reglib_errno;
  va_start(x, t);
  vsscanf(c, t, x);
  va_end(x);
  free(c);
  return reglib_errno;
}

int	regd_killkey(int sock, const char *path)
{
  regd_writetrans(sock, RGQ_KILLKEY, path, 0, 0, 0);
  return !reglib_errno;
}

int	regd_killval(int sock, const char *path)
{
  regd_writetrans(sock, RGQ_KILLVAL, path, 0, 0, 0);
  return !reglib_errno;
}

int	regd_login(int sock, char *aname, char *apwd)
{
  regd_writeint(sock, RGQ_LOGIN);
  regd_writestring(sock, aname);
  regd_writeint(sock, 0);
  regd_writestring(sock, apwd);
  reglib_errno = regd_readint(sock);
  return (reglib_errno == RG_ERR_SUCCESS);
}

int	regd_readint(int sock)
{
  int		x;
  reglib_errno	= RG_ERR_SUCCESS;
  if (read(sock, &x, sizeof(x)) < 0)
    reglib_errno	= errno;
  if (reglib_errno)
    return reglib_errno;
  return	x;
}

char *	regd_readstring(int sock)
{
  int	len;
  char *c;
  len		= regd_readint(sock);
  if (reglib_errno)
    return NULL;
  c		= (char *)malloc(len + 1);
  c[len]	= 0;
  read(sock, c, len);
  reglib_errno	= errno;
  return c;
}

int	regd_setkeyinfo(int sock, const char *path, int mode,
			int uid, int gid)
{
  int		data[2];
  if (uid == -1)
    regd_writetrans(sock, RGQ_SETKEYI, path, mode, 0, NULL);
  else
    {
      data[0]	= uid;
      data[1]	= gid;
      regd_writetrans(sock, RGQ_SETKEYI, path, mode, sizeof(data), data);
    }
  return reglib_errno;
}

int	regd_setkeymode(int sock, const char *path, int mode)
{
  regd_writetrans(sock, RGQ_SETKEYI, path, mode, 0, NULL);
  return reglib_errno;
}

int	regd_setval(int sock, const char *path, const char *val)
{
  regd_writetrans(sock, RGQ_SETVAL, path, 0, strlen(val) + 1, val);
  return reglib_errno;
}

int	regd_setvalv(int sock, const char *path, const char *valname,
		     const char *val)
{
  char *	c;
  int	x;
  c	= (char *)malloc(strlen(path) + strlen(valname) + 2);
  strcpy(c, path);
  if (valname[0] != '|')
    strcat(c, "|");
  strcat(c, valname);
  x	= regd_setval(sock, c, val);
  free(c);
  return x;
}

int	regd_setvals(int sock, const char *path, const char *t, ...)
{
  va_list	x;
  char *	c;
  va_start(x, t);
  c	= 0;
  vasprintf(&c, t, x);
  va_end(x);
  return regd_setval(sock, path, c);
}

int	regd_setvalsv(int sock, const char *path, 
		      const char *valname, const char *t, ...)
{
  va_list	x;
  char *	c;
  va_start(x, t);
  c	= 0;
  vasprintf(&c, t, x);
  va_end(x);
  return regd_setvalv(sock, path, valname, c);
}

int	regd_setvalr(int sock, const char *path, const char *val)
{
  regd_createkeyr(sock, path, RKM_DEF);
  regd_writetrans(sock, RGQ_SETVAL, path, 0, strlen(val) + 1, val);
  return reglib_errno;
}

int	regd_setvalvr(int sock, const char *path, const char *valname,
		     const char *val)
{
  char *	c;
  int	x;
  c	= (char *)alloca(strlen(path) + strlen(valname) + 2);
  strcpy(c, path);
  if (valname[0] != '|')
    strcat(c, "|");
  strcat(c, valname);
  x	= regd_setvalr(sock, c, val);
  return x;
}

int	regd_setvalsr(int sock, const char *path, const char *t, ...)
{
  va_list	x;
  char *	c;
  va_start(x, t);
  c	= 0;
  vasprintf(&c, t, x);
  va_end(x);
  return regd_setvalr(sock, path, c);
}

int	regd_setvalsvr(int sock, const char *path, 
		      const char *valname, const char *t, ...)
{
  va_list	x;
  char *	c;
  va_start(x, t);
  c	= 0;
  vasprintf(&c, t, x);
  va_end(x);
  return regd_setvalvr(sock, path, valname, c);
}

void	regd_writeint(int sock, int val)
{
  write(sock, &val, sizeof(val));
  reglib_errno	= errno;
}

void	regd_writestring(int sock, const char *string)
{
  int	x;

  x	= strlen(string);
  regd_writeint(sock, x + 1);
  write(sock, string, x + 1);
  reglib_errno	= errno;
}

void	regd_writetrans(int sock, int req, const char *name, int type, int len,
			const void *data)
{
  errno	= 0;
  regd_writeint(sock, req);
  regd_writestring(sock, name);
  regd_writeint(sock, type);
  if (len > 0)
    {
      regd_writeint(sock, len);
      write(sock, data, len);
    }
  else
    {
      regd_writeint(sock, 1);
      write(sock, "", 1);
    }
  reglib_errno	= regd_readint(sock);
}
