/* key.cc
 * Written by Joshua Rowe
 */

#include	"key.hh"
#include	"string.h"
#include	<grp.h>
#include	<pwd.h>

RegKey::RegKey(char *aname, int auid, int agid,
	       int amode, pRegKey aparent)
{
  init();
  if (aname != 0)		// If a name is provided, duplicate it
    {
      name	= new char[strlen(aname) + 1];
      strcpy(name, aname);
    }
  if (aparent != 0)		// Link into our parent's list
    {
      parent		= aparent;
      next		= parent->children;
      if (next)
	next->prev	= this;
      parent->children	= this;
    }
  uid	= auid;
  gid	= agid;
  mode	= amode;
}

RegKey::RegKey(pstreamableInit)
{
  init();			// I know we aren't supposed to do anything
				// here, but I couldn't resist.
}

RegKey::~RegKey()
{
  if (name)
    delete	name;		// Delete our name string if it exists
  if (prev == 0)
    {
      if (parent != 0)
	parent->children	= next;	// Unlink from parent if we are first in chain
    }  
  else
    prev->next	= next;		// Unlink from prev if not first
  if (next != 0)
    next->prev	= prev;		// Unlink from next if not last
  while (children)		// Delete each of our children
    delete children;		// children gets updated after each delete
  while (vals)			// Delete each of our values
    delete vals;		// vals gets updated after each delete
}

int		RegKey::access(int auid, int agid, int amode)
{
  if (auid == 0)
    return 1;
  if (parent != 0)
    if (!parent->access(auid, agid, RKM_X))
      return	0;
  if ((auid == uid) && (mode & (amode << RKM_U)) && (mode & RKM_UX))
    return 1;
  if ((agid == gid) && (mode & (amode << RKM_G)) && (mode & RKM_GX))
    return 1;
  if ((mode & (amode << RKM_O)) && (mode & RKM_OX))
    return 1;
  if (gid == -1)
    return 0;
  struct passwd *	p	= getpwuid(auid);
  struct group *	g	= getgrgid(gid);
  int	flag	= 1;
  for(char **c = g->gr_mem; (*c != 0) && flag; c++)
    flag	= strcmp(*c, p->pw_name);
  if (flag)
    return 0;
  if ((mode & (amode << RKM_G)) && (mode & RKM_GX))
    return 1;
  return 0;
}

pstreamable *	RegKey::build()
{
  return new RegKey(pstreamableinit); // Create a new clean RegKey
}

char *	RegKey::getfullname()
{
  char *	p;
  if (parent)
    p	= parent->getfullname();
  else
    {
      p		= new char[1];
      p[0]	= 0;
    }
  if (name == 0)
    return p;
  char *	q	= new char[strlen(p) + strlen(name) + 2];
  strcpy(q, p);
// strcat(q, "/");      The name includes the prefix '/'.
  strcat(q, name);
  delete	p;
  return q;
}

void	RegKey::init()
{
  name		= 0;
  parent	= 0;
  children	= 0;
  next		= 0;
  prev		= 0;
  vals		= 0;
}

pvoid	RegKey::read(ipstream &is) // Read ourselves from a stream
{
  is >> name >> uid >> gid >> mode
    >> parent >> children >> next >> prev >> vals;
  return this;
}

void	RegKey::write(opstream &os) // Write ourselves to a stream
{
  os << name << uid << gid << mode
    << parent << children << next << prev << vals;
}

pstreamreg	RegRegKey("RegKey", RegKey::build, __PSTREAM_DELTA(RegKey));

