/* mkdb.cc
 * Written by Joshua Rowe
 */

#include	"key.hh"
#include	"fpios.hh"
#include	<getopt.h>
#include	<stdio.h>
#include	<unistd.h>

int		verbose		= 1;
char *		regfn		= 0;
pRegKey		registry	= 0;
pRegKey		users		= 0;
pRegKey		defaultu	= 0;
pRegKey		systemk		= 0;

#define	OPT_VERSION	1000

char	buf[32768];

void	createregistry()
{
  if (verbose)
    printf("Creating new registry...\n");
  registry	= new RegKey("");
  users		= new RegKey("/users", 0, 0, RKM_DEF, registry);
  defaultu	= new RegKey("/default", 0, 0, RKM_DEF, users);
  systemk	= new RegKey("/system", 0, 0, RKM_DEF, registry);
  gethostname(buf, 32768);
  new RegVal("|hostname", buf, systemk);
  if (verbose)
    printf("Done.\n");
}

void	deleteregistry()
{
  if (verbose)
    printf("Cleaning up...\n");
  delete registry;
  if (verbose)
    printf("Done.\n");
}

void	printusage(char *arg0)
{
  fprintf(stderr,
	  "usage - %s [OPTION]...\n"
	  "\n"
	  "  -f, --file F          use file F\n"
	  "  -h, --help            output this screen and exit\n"
	  "  -q, --quiet           quiet (no output)\n"
          "  -v, --verbose         verbose (default)\n"
	  "      --version         output version information and exit\n"
	  "\n"
	  "Without a --file, " REGDB " is used instead.\n"
	  ,arg0
	  );
}

void	printversion()
{
  printf("mkdb version " VERNUM "\nWritten by Joshua Rowe.\n");
}

void	writeregistry()
{
  ofpstream	os(regfn, 
		   pstream::xxcreat | 
		   pstream::xxclose | 
		   pstream::xxwrite |
		   pstream::xxtrunc);
  if (verbose)
    printf("Writing to %s...\n", regfn);
  os << registry << users << defaultu << systemk;
  if (verbose)
    printf("Done.\n");
}

int	main(int argc, char **argv)
{
  regfn	= new char[strlen(REGDB) + 1];
  strcpy(regfn, REGDB);
  int	c;
  while (1)
    {
      static struct option long_options[] =
	{
	  {"file",	required_argument,	0,	'f'},
	  {"help",	no_argument,		0,	'h'},
	  {"quiet",	no_argument,		0,	'q'},
	  {"verbose",	no_argument,		0,	'v'},
	  {"version",	no_argument,		0,	OPT_VERSION},
	  {0,		0,			0,	0}
	};
      c	= getopt_long(argc, argv, "f:hqv", long_options, 0);
      if (c == -1)
	break;			// We have reached the end of the options
      switch(c)
	{
	case 'f':		// --file
	  delete	regfn;
	  regfn		= new char[strlen(optarg) + 1];
	  strcpy(regfn, optarg);
	  break;
	case 'h':		// --help
	  printusage(argv[0]);
	  return 0;
	case 'q':		// --quiet
	  verbose	= 0;
	  break;
	case 'v':
	  verbose	= 1;
	  break;
	case OPT_VERSION:	// --version
	  printversion();
	  return 0;
	case '?':		// invalid option
	  printusage(argv[0]);
	  return 1;
	}
    }
  createregistry();		// Create the new registry
  writeregistry();		// Write it to the registry file
  deleteregistry();		// Delete the old one
				// In reality, this isn't necessary.
				// I like completion.
  return 0;
}
